@@ -66,7 +66,7 @@ public class CoyoteAdapter implements Adapter {
 
     // -------------------------------------------------------------- Constants
 
-    private static final String POWERED_BY = "Servlet/3.1 JSP/2.3 " + "(" + ServerInfo.getServerInfo() + " Java/" +
+    private static final String POWERED_BY = "Servlet/4.0 JSP/2.3 " + "(" + ServerInfo.getServerInfo() + " Java/" +
             System.getProperty("java.vm.vendor") + "/" + System.getProperty("java.runtime.version") + ")";
 
     private static final EnumSet<SessionTrackingMode> SSL_ONLY = EnumSet.of(SessionTrackingMode.SSL);
@@ -139,7 +139,7 @@ public class CoyoteAdapter implements Adapter {
                 }
             } else if (status == SocketEvent.ERROR) {
                 // An I/O error occurred on a non-container thread which means
-                // that the socket needs to be closed so set success to false to
+                // that the socket needs to be closed so set success as false to
                 // trigger a close
                 success = false;
                 Throwable t = (Throwable) req.getAttribute(RequestDispatcher.ERROR_EXCEPTION);
@@ -179,6 +179,8 @@ public class CoyoteAdapter implements Adapter {
                         }
                     } catch (Throwable t) {
                         ExceptionUtils.handleThrowable(t);
+                        // Allow the error handling to write to the response
+                        response.setSuspended(false);
                         // Need to trigger the call to AbstractProcessor.setErrorState()
                         // before the listener is called so the listener can call complete
                         // Therefore no need to set success=false as that would trigger a
@@ -212,6 +214,8 @@ public class CoyoteAdapter implements Adapter {
                         }
                     } catch (Throwable t) {
                         ExceptionUtils.handleThrowable(t);
+                        // Allow the error handling to write to the response
+                        response.setSuspended(false);
                         // Need to trigger the call to AbstractProcessor.setErrorState()
                         // before the listener is called so the listener can call complete
                         // Therefore no need to set success=false as that would trigger a
@@ -235,8 +239,8 @@ public class CoyoteAdapter implements Adapter {
 
             if (request.isAsyncDispatching()) {
                 connector.getService().getContainer().getPipeline().getFirst().invoke(request, response);
-                Throwable t = (Throwable) request.getAttribute(RequestDispatcher.ERROR_EXCEPTION);
-                if (t != null) {
+                if (response.isError()) {
+                    Throwable t = (Throwable) request.getAttribute(RequestDispatcher.ERROR_EXCEPTION);
                     asyncConImpl.setErrorState(t, true);
                 }
             }
@@ -251,10 +255,9 @@ public class CoyoteAdapter implements Adapter {
             AtomicBoolean error = new AtomicBoolean(false);
             res.action(ActionCode.IS_ERROR, error);
             if (error.get()) {
-                if (request.isAsyncCompleting()) {
-                    // Connection will be forcibly closed which will prevent
-                    // completion happening at the usual point. Need to trigger
-                    // call to onComplete() here.
+                if (request.isAsyncCompleting() || request.isAsyncDispatching()) {
+                    // Connection will be forcibly closed which will prevent completion/dispatch happening at the usual
+                    // point. Trigger post-processing here.
                     res.action(ActionCode.ASYNC_POST_PROCESS, null);
                 }
                 success = false;
@@ -555,6 +558,7 @@ public class CoyoteAdapter implements Adapter {
      * @throws IOException      If there is insufficient space in a buffer while processing headers
      * @throws ServletException If the supported methods of the target servlet cannot be determined
      */
+    @SuppressWarnings("deprecation")
     protected boolean postParseRequest(org.apache.coyote.Request req, Request request, org.apache.coyote.Response res,
             Response response) throws IOException, ServletException {
 
@@ -593,7 +597,7 @@ public class CoyoteAdapter implements Adapter {
 
         // Check for ping OPTIONS * request
         if (undecodedURI.equals("*")) {
-            if (req.method().equalsIgnoreCase("OPTIONS")) {
+            if (req.method().equals("OPTIONS")) {
                 StringBuilder allow = new StringBuilder();
                 allow.append("GET, HEAD, POST, PUT, DELETE, OPTIONS");
                 // Trace if allowed
@@ -612,7 +616,7 @@ public class CoyoteAdapter implements Adapter {
         MessageBytes decodedURI = req.decodedURI();
 
         // Filter CONNECT method
-        if (req.method().equalsIgnoreCase("CONNECT")) {
+        if (req.method().equals("CONNECT")) {
             response.sendError(HttpServletResponse.SC_NOT_IMPLEMENTED, sm.getString("coyoteAdapter.connect"));
         } else {
             // No URI for CONNECT requests
@@ -627,7 +631,8 @@ public class CoyoteAdapter implements Adapter {
                 // %xx decoding of the URL
                 try {
                     req.getURLDecoder().convert(decodedURI.getByteChunk(),
-                            connector.getEncodedSolidusHandlingInternal());
+                            connector.getEncodedSolidusHandlingInternal(),
+                            connector.getEncodedReverseSolidusHandlingInternal());
                 } catch (IOException ioe) {
                     response.sendError(400, sm.getString("coyoteAdapter.invalidURIWithMessage", ioe.getMessage()));
                 }
@@ -636,6 +641,9 @@ public class CoyoteAdapter implements Adapter {
                     // Character decoding
                     convertURI(decodedURI, request);
                     // Check that the URI is still normalized
+                    // Note: checkNormalize is deprecated because the test is no
+                    // longer required in Tomcat 10 onwards and has been
+                    // removed
                     if (!checkNormalize(req.decodedURI())) {
                         response.sendError(400, "Invalid URI");
                     }
@@ -645,17 +653,17 @@ public class CoyoteAdapter implements Adapter {
             } else {
                 /*
                  * The URI is chars or String, and has been sent using an in-memory protocol handler. The following
-                 * assumptions are made: - req.requestURI() has been set to the 'original' non-decoded, non-normalized
-                 * URI - req.decodedURI() has been set to the decoded, normalized form of req.requestURI()
+                 * assumptions are made:
+                 *
+                 * - req.requestURI() has been set to the 'original' non-decoded, non-normalized URI that includes path
+                 * parameters (if any)
+                 *
+                 * - req.decodedURI() has been set to the decoded, normalized form of req.requestURI() with any path
+                 * parameters removed
+                 *
+                 * - 'suspicious' URI filtering, if required, has already been performed
                  */
                 decodedURI.toChars();
-                // Remove all path parameters; any needed path parameter should be set
-                // using the request object rather than passing it in the URL
-                CharChunk uriCC = decodedURI.getCharChunk();
-                int semicolon = uriCC.indexOf(';');
-                if (semicolon > 0) {
-                    decodedURI.setChars(uriCC.getBuffer(), uriCC.getStart(), semicolon);
-                }
             }
         }
 
@@ -685,7 +693,7 @@ public class CoyoteAdapter implements Adapter {
         }
 
         while (mapRequired) {
-            // This will map the the latest version by default
+            // This will map the latest version by default
             connector.getService().getMapper().map(serverName, decodedURI, version, request.getMappingData());
 
             // If there is no context at this point, either this is a 404
@@ -805,9 +813,9 @@ public class CoyoteAdapter implements Adapter {
         }
 
         // Filter TRACE method
-        if (!connector.getAllowTrace() && req.method().equalsIgnoreCase("TRACE")) {
+        if (!connector.getAllowTrace() && req.method().equals("TRACE")) {
             Wrapper wrapper = request.getWrapper();
-            String header = null;
+            StringBuilder header = null;
             if (wrapper != null) {
                 String[] methods = wrapper.getServletMethods();
                 if (methods != null) {
@@ -816,15 +824,15 @@ public class CoyoteAdapter implements Adapter {
                             continue;
                         }
                         if (header == null) {
-                            header = method;
+                            header = new StringBuilder(method);
                         } else {
-                            header += ", " + method;
+                            header.append(", ").append(method);
                         }
                     }
                 }
             }
             if (header != null) {
-                res.addHeader("Allow", header);
+                res.addHeader("Allow", header.toString());
             }
             response.sendError(405, sm.getString("coyoteAdapter.trace"));
             // Safe to skip the remainder of this method.
@@ -860,7 +868,7 @@ public class CoyoteAdapter implements Adapter {
                 // result preventing excessive calls to the Realm.
             } else {
                 // The connector isn't configured for authorization. Create a
-                // user without any roles using the supplied user name.
+                // user without any roles using the supplied username.
                 request.setUserPrincipal(new CoyotePrincipal(username));
             }
         }
@@ -901,10 +909,10 @@ public class CoyoteAdapter implements Adapter {
         // encoding that doesn't give the expected result so be explicit
         Charset charset = connector.getURICharset();
 
-        if (log.isDebugEnabled()) {
-            log.debug(sm.getString("coyoteAdapter.debug", "uriBC", uriBC.toString()));
-            log.debug(sm.getString("coyoteAdapter.debug", "semicolon", String.valueOf(semicolon)));
-            log.debug(sm.getString("coyoteAdapter.debug", "enc", charset.name()));
+        if (log.isTraceEnabled()) {
+            log.trace(sm.getString("coyoteAdapter.debug", "uriBC", uriBC.toString()));
+            log.trace(sm.getString("coyoteAdapter.debug", "semicolon", String.valueOf(semicolon)));
+            log.trace(sm.getString("coyoteAdapter.debug", "enc", charset.name()));
         }
 
         while (semicolon > -1) {
@@ -935,10 +943,10 @@ public class CoyoteAdapter implements Adapter {
                 uriBC.setEnd(start + semicolon);
             }
 
-            if (log.isDebugEnabled()) {
-                log.debug(sm.getString("coyoteAdapter.debug", "pathParamStart", String.valueOf(pathParamStart)));
-                log.debug(sm.getString("coyoteAdapter.debug", "pathParamEnd", String.valueOf(pathParamEnd)));
-                log.debug(sm.getString("coyoteAdapter.debug", "pv", pv));
+            if (log.isTraceEnabled()) {
+                log.trace(sm.getString("coyoteAdapter.debug", "pathParamStart", String.valueOf(pathParamStart)));
+                log.trace(sm.getString("coyoteAdapter.debug", "pathParamEnd", String.valueOf(pathParamEnd)));
+                log.trace(sm.getString("coyoteAdapter.debug", "pv", pv));
             }
 
             if (pv != null) {
@@ -947,10 +955,10 @@ public class CoyoteAdapter implements Adapter {
                     String name = pv.substring(0, equals);
                     String value = pv.substring(equals + 1);
                     request.addPathParameter(name, value);
-                    if (log.isDebugEnabled()) {
-                        log.debug(sm.getString("coyoteAdapter.debug", "equals", String.valueOf(equals)));
-                        log.debug(sm.getString("coyoteAdapter.debug", "name", name));
-                        log.debug(sm.getString("coyoteAdapter.debug", "value", value));
+                    if (log.isTraceEnabled()) {
+                        log.trace(sm.getString("coyoteAdapter.debug", "equals", String.valueOf(equals)));
+                        log.trace(sm.getString("coyoteAdapter.debug", "name", name));
+                        log.trace(sm.getString("coyoteAdapter.debug", "value", value));
                     }
                 }
             }
@@ -1014,8 +1022,8 @@ public class CoyoteAdapter implements Adapter {
                     request.setRequestedSessionId(scookie.getValue().toString());
                     request.setRequestedSessionCookie(true);
                     request.setRequestedSessionURL(false);
-                    if (log.isDebugEnabled()) {
-                        log.debug(" Requested cookie session id is " + request.getRequestedSessionId());
+                    if (log.isTraceEnabled()) {
+                        log.trace(" Requested cookie session id is " + request.getRequestedSessionId());
                     }
                 } else {
                     if (!request.isRequestedSessionIdValid()) {
@@ -1067,7 +1075,7 @@ public class CoyoteAdapter implements Adapter {
 
 
     /**
-     * Character conversion of the a US-ASCII MessageBytes.
+     * Character conversion of the US-ASCII MessageBytes.
      *
      * @param mb The MessageBytes instance containing the bytes that should be converted to chars
      */
@@ -1115,15 +1123,13 @@ public class CoyoteAdapter implements Adapter {
             return false;
         }
 
-        int pos = 0;
-        int index = 0;
-
-
         // The URL must start with '/' (or '\' that will be replaced soon)
         if (b[start] != (byte) '/' && b[start] != (byte) '\\') {
             return false;
         }
 
+        int pos;
+
         // Replace '\' with '/'
         // Check for null byte
         for (pos = start; pos < end; pos++) {
@@ -1160,7 +1166,7 @@ public class CoyoteAdapter implements Adapter {
 
         uriBC.setEnd(end);
 
-        index = 0;
+        int index = 0;
 
         // Resolve occurrences of "/./" in the normalized path
         while (true) {
@@ -1210,7 +1216,10 @@ public class CoyoteAdapter implements Adapter {
      *
      * @return <code>false</code> if sequences that are supposed to be normalized are still present in the URI,
      *             otherwise <code>true</code>
+     *
+     * @deprecated This code will be removed in Apache Tomcat 10 onwards
      */
+    @Deprecated
     public static boolean checkNormalize(MessageBytes uriMB) {
 
         CharChunk uriCC = uriMB.getCharChunk();
