@@ -91,6 +91,7 @@ public class Response implements HttpServletResponse {
                 System.getProperty("org.apache.catalina.connector.Response.ENFORCE_ENCODING_IN_GET_WRITER", "true"));
     }
 
+    protected static final int SC_EARLY_HINTS = 103;
 
     // ----------------------------------------------------- Instance Variables
 
@@ -313,7 +314,7 @@ public class Response implements HttpServletResponse {
     /**
      * @return the Request with which this Response is associated.
      */
-    public org.apache.catalina.connector.Request getRequest() {
+    public Request getRequest() {
         return this.request;
     }
 
@@ -322,7 +323,7 @@ public class Response implements HttpServletResponse {
      *
      * @param request The new associated request
      */
-    public void setRequest(org.apache.catalina.connector.Request request) {
+    public void setRequest(Request request) {
         this.request = request;
     }
 
@@ -400,7 +401,10 @@ public class Response implements HttpServletResponse {
      * Set the error flag.
      *
      * @return <code>false</code> if the error flag was already set
+     *
+     * @deprecated This method will be changed to return void in Tomcat 11 onwards
      */
+    @Deprecated
     public boolean setError() {
         return getCoyoteResponse().setError();
     }
@@ -426,6 +430,11 @@ public class Response implements HttpServletResponse {
     }
 
 
+    public void resetError() {
+        getCoyoteResponse().resetError();
+    }
+
+
     /**
      * Perform whatever actions are required to flush and close the output stream or writer, in a single operation.
      *
@@ -445,10 +454,6 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * @return the content type that was set or calculated for this response, or <code>null</code> if no content type
-     *             was set.
-     */
     @Override
     public String getContentType() {
         return getCoyoteResponse().getContentType();
@@ -481,29 +486,18 @@ public class Response implements HttpServletResponse {
     // ------------------------------------------------ ServletResponse Methods
 
 
-    /**
-     * Flush the buffer and commit this response.
-     *
-     * @exception IOException if an input/output error occurs
-     */
     @Override
     public void flushBuffer() throws IOException {
         outputBuffer.flush();
     }
 
 
-    /**
-     * @return the actual buffer size used for this Response.
-     */
     @Override
     public int getBufferSize() {
         return outputBuffer.getBufferSize();
     }
 
 
-    /**
-     * @return the character encoding used for this Response.
-     */
     @Override
     public String getCharacterEncoding() {
         String charset = getCoyoteResponse().getCharacterEncoding();
@@ -525,12 +519,6 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * @return the servlet output stream associated with this Response.
-     *
-     * @exception IllegalStateException if <code>getWriter</code> has already been called for this response
-     * @exception IOException           if an input/output error occurs
-     */
     @Override
     public ServletOutputStream getOutputStream() throws IOException {
 
@@ -547,21 +535,12 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * @return the Locale assigned to this response.
-     */
     @Override
     public Locale getLocale() {
         return getCoyoteResponse().getLocale();
     }
 
 
-    /**
-     * @return the writer associated with this Response.
-     *
-     * @exception IllegalStateException if <code>getOutputStream</code> has already been called for this response
-     * @exception IOException           if an input/output error occurs
-     */
     @Override
     public PrintWriter getWriter() throws IOException {
 
@@ -590,22 +569,12 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * Has the output of this response already been committed?
-     *
-     * @return <code>true</code> if the response has been committed
-     */
     @Override
     public boolean isCommitted() {
         return getCoyoteResponse().isCommitted();
     }
 
 
-    /**
-     * Clear any content written to the buffer.
-     *
-     * @exception IllegalStateException if this response has already been committed
-     */
     @Override
     public void reset() {
         // Ignore any call from an included servlet
@@ -621,11 +590,6 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * Reset the data buffer but not any status or header information.
-     *
-     * @exception IllegalStateException if the response has already been committed
-     */
     @Override
     public void resetBuffer() {
         resetBuffer(false);
@@ -658,13 +622,6 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * Set the buffer size to be used for this Response.
-     *
-     * @param size The new buffer size
-     *
-     * @exception IllegalStateException if this method is called after output has been committed for this response
-     */
     @Override
     public void setBufferSize(int size) {
 
@@ -677,11 +634,6 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * Set the content length (in bytes) for this Response.
-     *
-     * @param length The new content length
-     */
     @Override
     public void setContentLength(int length) {
 
@@ -704,11 +656,6 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * Set the content type for this Response.
-     *
-     * @param type The new content type
-     */
     @Override
     public void setContentType(String type) {
 
@@ -765,12 +712,6 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * Overrides the name of the character encoding used in the body of the request. This method must be called prior to
-     * reading request parameters or reading input using getReader().
-     *
-     * @param encoding String containing the name of the character encoding.
-     */
     @Override
     public void setCharacterEncoding(String encoding) {
 
@@ -795,19 +736,10 @@ public class Response implements HttpServletResponse {
             log.warn(sm.getString("coyoteResponse.encoding.invalid", encoding), e);
             return;
         }
-        if (encoding == null) {
-            isCharacterEncodingSet = false;
-        } else {
-            isCharacterEncodingSet = true;
-        }
+        isCharacterEncodingSet = encoding != null;
     }
 
 
-    /**
-     * Set the Locale that is appropriate for this response, including setting the appropriate character encoding.
-     *
-     * @param locale The new locale
-     */
     @Override
     public void setLocale(Locale locale) {
 
@@ -972,16 +904,10 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * Add the specified date header to the specified value.
-     *
-     * @param name  Name of the header to set
-     * @param value Date value to be set
-     */
     @Override
     public void addDateHeader(String name, long value) {
 
-        if (name == null || name.length() == 0) {
+        if (name == null || name.isEmpty()) {
             return;
         }
 
@@ -998,12 +924,6 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * Add the specified header to the specified value.
-     *
-     * @param name  Name of the header to set
-     * @param value Value to be set
-     */
     @Override
     public void addHeader(String name, String value) {
         addHeader(name, value, null);
@@ -1012,7 +932,7 @@ public class Response implements HttpServletResponse {
 
     private void addHeader(String name, String value, Charset charset) {
 
-        if (name == null || name.length() == 0 || value == null) {
+        if (name == null || name.isEmpty() || value == null) {
             return;
         }
 
@@ -1052,16 +972,10 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * Add the specified integer header to the specified value.
-     *
-     * @param name  Name of the header to set
-     * @param value Integer value to be set
-     */
     @Override
     public void addIntHeader(String name, int value) {
 
-        if (name == null || name.length() == 0) {
+        if (name == null || name.isEmpty()) {
             return;
         }
 
@@ -1079,13 +993,6 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * Has the specified header been set already in this response?
-     *
-     * @param name Name of the header to check
-     *
-     * @return <code>true</code> if the header has been set
-     */
     @Override
     public boolean containsHeader(String name) {
         // Need special handling for Content-Type and Content-Length due to
@@ -1118,13 +1025,6 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * Encode the session identifier associated with this response into the specified redirect URL, if necessary.
-     *
-     * @param url URL to be encoded
-     *
-     * @return <code>true</code> if the URL was encoded
-     */
     @Override
     public String encodeRedirectURL(String url) {
         if (isEncodeable(toAbsolute(url))) {
@@ -1135,15 +1035,6 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * Encode the session identifier associated with this response into the specified redirect URL, if necessary.
-     *
-     * @param url URL to be encoded
-     *
-     * @return <code>true</code> if the URL was encoded
-     *
-     * @deprecated As of Version 2.1 of the Java Servlet API, use <code>encodeRedirectURL()</code> instead.
-     */
     @Override
     @Deprecated
     public String encodeRedirectUrl(String url) {
@@ -1151,13 +1042,6 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * Encode the session identifier associated with this response into the specified URL, if necessary.
-     *
-     * @param url URL to be encoded
-     *
-     * @return <code>true</code> if the URL was encoded
-     */
     @Override
     public String encodeURL(String url) {
 
@@ -1236,13 +1120,29 @@ public class Response implements HttpServletResponse {
     }
 
 
+    public void sendEarlyHints() {
+        if (isCommitted()) {
+            return;
+        }
+
+        // Ignore any call from an included servlet
+        if (included) {
+            return;
+        }
+
+        getCoyoteResponse().action(ActionCode.EARLY_HINTS, null);
+    }
+
+
     /**
-     * Send an error response with the specified status and a default message.
-     *
-     * @param status HTTP status code to send
-     *
-     * @exception IllegalStateException if this response has already been committed
-     * @exception IOException           if an input/output error occurs
+     * {@inheritDoc}
+     * <p>
+     * Calling <code>sendError</code> with a status code of 103 differs from the usual behavior. Sending 103 will
+     * trigger the container to send a "103 Early Hints" informational response including all current headers. The
+     * application can continue to use the request and response after calling sendError with a 103 status code,
+     * including triggering a more typical response of any type.
+     * <p>
+     * Starting with Tomcat 12, applications should use {@link #sendEarlyHints}.
      */
     @Override
     public void sendError(int status) throws IOException {
@@ -1251,13 +1151,14 @@ public class Response implements HttpServletResponse {
 
 
     /**
-     * Send an error response with the specified status and message.
-     *
-     * @param status  HTTP status code to send
-     * @param message Corresponding message to send
-     *
-     * @exception IllegalStateException if this response has already been committed
-     * @exception IOException           if an input/output error occurs
+     * {@inheritDoc}
+     * <p>
+     * Calling <code>sendError</code> with a status code of 103 differs from the usual behavior. Sending 103 will
+     * trigger the container to send a "103 Early Hints" informational response including all current headers. The
+     * application can continue to use the request and response after calling sendError with a 103 status code,
+     * including triggering a more typical response of any type.
+     * <p>
+     * Starting with Tomcat 12, applications should use {@link #sendEarlyHints}.
      */
     @Override
     public void sendError(int status, String message) throws IOException {
@@ -1271,16 +1172,20 @@ public class Response implements HttpServletResponse {
             return;
         }
 
-        setError();
+        if (SC_EARLY_HINTS == status) {
+            sendEarlyHints();
+        } else {
+            setError();
 
-        getCoyoteResponse().setStatus(status);
-        getCoyoteResponse().setMessage(message);
+            getCoyoteResponse().setStatus(status);
+            getCoyoteResponse().setMessage(message);
 
-        // Clear any data content that has been buffered
-        resetBuffer();
+            // Clear any data content that has been buffered
+            resetBuffer();
 
-        // Cause the response to be finished (from the application perspective)
-        setSuspended(true);
+            // Cause the response to be finished (from the application perspective)
+            setSuspended(true);
+        }
     }
 
 
@@ -1343,16 +1248,10 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * Set the specified date header to the specified value.
-     *
-     * @param name  Name of the header to set
-     * @param value Date value to be set
-     */
     @Override
     public void setDateHeader(String name, long value) {
 
-        if (name == null || name.length() == 0) {
+        if (name == null || name.isEmpty()) {
             return;
         }
 
@@ -1369,16 +1268,10 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * Set the specified header to the specified value.
-     *
-     * @param name  Name of the header to set
-     * @param value Value to be set
-     */
     @Override
     public void setHeader(String name, String value) {
 
-        if (name == null || name.length() == 0 || value == null) {
+        if (name == null || name.isEmpty() || value == null) {
             return;
         }
 
@@ -1402,16 +1295,10 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * Set the specified integer header to the specified value.
-     *
-     * @param name  Name of the header to set
-     * @param value Integer value to be set
-     */
     @Override
     public void setIntHeader(String name, int value) {
 
-        if (name == null || name.length() == 0) {
+        if (name == null || name.isEmpty()) {
             return;
         }
 
@@ -1429,11 +1316,6 @@ public class Response implements HttpServletResponse {
     }
 
 
-    /**
-     * Set the HTTP status to be returned with this response.
-     *
-     * @param status The new HTTP status
-     */
     @Override
     public void setStatus(int status) {
         setStatus(status, null);
@@ -1521,7 +1403,7 @@ public class Response implements HttpServletResponse {
 
     private static boolean doIsEncodeable(Context context, Request hreq, Session session, String location) {
         // Is this a valid absolute URL?
-        URL url = null;
+        URL url;
         try {
             URI uri = new URI(location);
             url = uri.toURL();
@@ -1563,9 +1445,7 @@ public class Response implements HttpServletResponse {
                 return false;
             }
             String tok = ";" + SessionConfig.getSessionUriParamName(context) + "=" + session.getIdInternal();
-            if (file.indexOf(tok, contextPath.length()) >= 0) {
-                return false;
-            }
+            return file.indexOf(tok, contextPath.length()) < 0;
         }
 
         // This URL belongs to our web application, so it is encodeable
@@ -1588,7 +1468,7 @@ public class Response implements HttpServletResponse {
     protected String toAbsolute(String location) {
 
         if (location == null) {
-            return location;
+            return null;
         }
 
         boolean leadingSlash = location.startsWith("/");
@@ -1688,7 +1568,6 @@ public class Response implements HttpServletResponse {
         char[] c = cc.getChars();
         int start = cc.getStart();
         int end = cc.getEnd();
-        int index = 0;
         int startIndex = 0;
 
         // Advance past the first three / characters (should place index just
@@ -1699,7 +1578,7 @@ public class Response implements HttpServletResponse {
         }
 
         // Remove /./
-        index = startIndex;
+        int index = startIndex;
         while (true) {
             index = cc.indexOf("/./", 0, 3, index);
             if (index < 0) {
@@ -1762,10 +1641,7 @@ public class Response implements HttpServletResponse {
             return false;
         }
         pos = uri.indexOf('/', pos + 3);
-        if (pos < 0) {
-            return false;
-        }
-        return true;
+        return pos >= 0;
     }
 
     /**
@@ -1777,7 +1653,7 @@ public class Response implements HttpServletResponse {
      * @return the encoded URL
      */
     protected String toEncoded(String url, String sessionId) {
-        if ((url == null) || (sessionId == null)) {
+        if (url == null || sessionId == null) {
             return url;
         }
 
