@@ -17,6 +17,7 @@
 package org.apache.catalina.loader;
 
 import java.io.File;
+import java.io.FileNotFoundException;
 import java.io.FilePermission;
 import java.io.IOException;
 import java.io.InputStream;
@@ -384,7 +385,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
     /**
      * Repositories managed by this class rather than the super class.
      */
-    private List<URL> localRepositories = new ArrayList<>();
+    private final List<URL> localRepositories = new ArrayList<>();
 
 
     private volatile LifecycleState state = LifecycleState.NEW;
@@ -748,7 +749,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
             sb.append(this.parent.toString());
             sb.append("\r\n");
         }
-        if (this.transformers.size() > 0) {
+        if (!this.transformers.isEmpty()) {
             sb.append("----------> Class file transformers:\r\n");
             for (ClassFileTransformer transformer : this.transformers) {
                 sb.append(transformer).append("\r\n");
@@ -910,7 +911,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
             }
 
             // Skip caching results for invalid names (it might mask lookups for valid ones)
-            if (url == null && name.charAt(0) != '/') {
+            if (url == null && !name.isEmpty() && name.charAt(0) != '/') {
                 notFoundClassResources.add(path);
             }
         }
@@ -995,7 +996,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
 
         checkStateForResourceLoading(name);
 
-        URL url = null;
+        URL url;
 
         boolean delegateFirst = delegate || filter(name, false);
 
@@ -1122,7 +1123,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
                 return stream;
             }
             // Skip caching results for invalid names (it might mask lookups for valid ones)
-            if (name.charAt(0) != '/') {
+            if (!name.isEmpty() && name.charAt(0) != '/') {
                 notFoundClassResources.add(path);
             }
         }
@@ -1189,7 +1190,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
             if (log.isTraceEnabled()) {
                 log.trace("loadClass(" + name + ", " + resolve + ")");
             }
-            Class<?> clazz = null;
+            Class<?> clazz;
 
             // Log access to stopped class loader
             checkStateForClassLoading(name);
@@ -1624,6 +1625,9 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
         byte[] classBytes = new byte[2048];
         int offset = 0;
         try (InputStream is = getResourceAsStream("org/apache/catalina/loader/JdbcLeakPrevention.class")) {
+            if (is == null) {
+                throw new FileNotFoundException("org/apache/catalina/loader/JdbcLeakPrevention.class");
+            }
             int read = is.read(classBytes, offset, classBytes.length - offset);
             while (read > -1) {
                 offset += read;
@@ -1749,7 +1753,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
         // when the executor was shut down or the thread was interrupted but
         // that depends on the thread correctly handling the interrupt. Check
         // each thread and if any are still running give all threads up to a
-        // total of 2 seconds to shutdown.
+        // total of 2 seconds to shut down.
         int count = 0;
         for (Thread t : threadsToStop) {
             while (t.isAlive() && count < 100) {
@@ -1780,7 +1784,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
 
         StackTraceElement[] elements = thread.getStackTrace();
 
-        if (elements == null || elements.length == 0) {
+        if (elements.length == 0) {
             // Must have stopped already. Too late to ignore it. Assume not a
             // request processing thread.
             return false;
@@ -2025,7 +2029,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
     }
 
     /**
-     * @return the set of current threads as an array.
+     * @return the current threads as an array.
      */
     private Thread[] getThreads() {
         // Get the current thread group
@@ -2213,11 +2217,6 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
     }
 
 
-    /*
-     * The use of getPackage() is appropriate given that the code is checking if the package is sealed. Therefore,
-     * parent class loaders need to be checked.
-     */
-    @SuppressWarnings("deprecation")
     private Class<?> findClassInternal(String name, String path) {
         ResourceEntry entry = resourceEntries.get(path);
         WebResource resource = null;
@@ -2275,7 +2274,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
             URL codeBase = resource.getCodeBase();
             Certificate[] certificates = resource.getCertificates();
 
-            if (transformers.size() > 0) {
+            if (!transformers.isEmpty()) {
                 // If the resource is a class just being loaded, decorate it
                 // with any attached transformers
 
@@ -2328,7 +2327,11 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
                 if (pkg != null) {
                     boolean sealCheck = true;
                     if (pkg.isSealed()) {
-                        sealCheck = pkg.isSealed(codeBase);
+                        if(codeBase != null) {
+                            sealCheck = pkg.isSealed(codeBase);
+                        } else {
+                            sealCheck = false;
+                        }
                     } else {
                         sealCheck = manifest == null || !isPackageSealed(packageName, manifest);
                     }
@@ -2540,6 +2543,8 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
     protected void addURL(URL url) {
         super.addURL(url);
         hasExternalRepositories = true;
+        // Clear the not found resources as they may now be available at the added URL.
+        notFoundClassResources.clear();
     }
 
 
