@@ -269,7 +269,7 @@ public class HostManagerServlet extends HttpServlet implements ContainerServlet
         }
 
         // Set our properties from the initialization parameters
-        String value = null;
+        String value;
         try {
             value = getServletConfig().getInitParameter("debug");
             debug = Integer.parseInt(value);
@@ -306,7 +306,7 @@ public class HostManagerServlet extends HttpServlet implements ContainerServlet
         }
 
         // Validate the requested host name
-        if (name == null || name.length() == 0) {
+        if (name == null || name.isEmpty()) {
             writer.println(smClient.getString("hostManagerServlet.invalidHostName", name));
             return;
         }
@@ -318,10 +318,10 @@ public class HostManagerServlet extends HttpServlet implements ContainerServlet
         }
 
         // Validate and create appBase
-        File appBaseFile = null;
-        File file = null;
+        File appBaseFile;
+        File file;
         String applicationBase = appBase;
-        if (applicationBase == null || applicationBase.length() == 0) {
+        if (applicationBase == null || applicationBase.isEmpty()) {
             applicationBase = name;
         }
         file = new File(applicationBase);
@@ -412,7 +412,7 @@ public class HostManagerServlet extends HttpServlet implements ContainerServlet
         }
 
         // Validate the requested host name
-        if (name == null || name.length() == 0) {
+        if (name == null || name.isEmpty()) {
             writer.println(smClient.getString("hostManagerServlet.invalidHostName", name));
             return;
         }
@@ -490,7 +490,7 @@ public class HostManagerServlet extends HttpServlet implements ContainerServlet
         }
 
         // Validate the requested host name
-        if (name == null || name.length() == 0) {
+        if (name == null || name.isEmpty()) {
             writer.println(smClient.getString("hostManagerServlet.invalidHostName", name));
             return;
         }
@@ -541,7 +541,7 @@ public class HostManagerServlet extends HttpServlet implements ContainerServlet
         }
 
         // Validate the requested host name
-        if (name == null || name.length() == 0) {
+        if (name == null || name.isEmpty()) {
             writer.println(smClient.getString("hostManagerServlet.invalidHostName", name));
             return;
         }
