@@ -21,11 +21,13 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.StringReader;
+import java.net.URLDecoder;
 import java.nio.charset.Charset;
 import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.StringTokenizer;
 import java.util.concurrent.ConcurrentHashMap;
 
@@ -38,6 +40,7 @@ import org.apache.catalina.Context;
 import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.Pipeline;
+import org.apache.catalina.Valve;
 import org.apache.catalina.connector.Connector;
 import org.apache.catalina.connector.Request;
 import org.apache.catalina.connector.Response;
@@ -53,21 +56,34 @@ import org.apache.tomcat.util.file.ConfigurationSource;
 import org.apache.tomcat.util.http.RequestUtil;
 
 /**
- * Note: Extra caution should be used when adding a Rewrite Rule. When
- * specifying a regex to match for in a Rewrite Rule, certain regex could allow
- * an attacker to DoS your server, as Java's regex parsing is vulnerable to
- * "catastrophic backtracking" (also known as "Regular expression Denial of
- * Service", or ReDoS). There are some open source tools to help detect
- * vulnerable regex, though in general it is a hard problem. A good defence is
- * to use a regex debugger on your desired regex, and read more on the subject
- * of catastrophic backtracking.
+ * Note: Extra caution should be used when adding a Rewrite Rule. When specifying a regex to match for in a Rewrite
+ * Rule, certain regex could allow an attacker to DoS your server, as Java's regex parsing is vulnerable to
+ * "catastrophic backtracking" (also known as "Regular expression Denial of Service", or ReDoS). There are some open
+ * source tools to help detect vulnerable regex, though in general it is a hard problem. A good defence is to use a
+ * regex debugger on your desired regex, and read more on the subject of catastrophic backtracking.
  *
- * @see <a href=
- *      "https://www.owasp.org/index.php/Regular_expression_Denial_of_Service_-_ReDoS">OWASP
- *      ReDoS</a>
+ * @see <a href= "https://www.owasp.org/index.php/Regular_expression_Denial_of_Service_-_ReDoS">OWASP ReDoS</a>
  */
 public class RewriteValve extends ValveBase {
 
+    private static final URLEncoder REWRITE_DEFAULT_ENCODER;
+    private static final URLEncoder REWRITE_QUERY_ENCODER;
+
+    static {
+        /*
+         * See the detailed explanation of encoding/decoding during URL re-writing in the invoke() method.
+         *
+         * These encoders perform the second stage of encoding, after re-writing has completed. These rewrite specific
+         * encoders treat '%' as a safe character so that URLs and query strings already processed by encodeForRewrite()
+         * do not end up with double encoding of '%' characters.
+         */
+        REWRITE_DEFAULT_ENCODER = (URLEncoder) URLEncoder.DEFAULT.clone();
+        REWRITE_DEFAULT_ENCODER.addSafeCharacter('%');
+
+        REWRITE_QUERY_ENCODER = (URLEncoder) URLEncoder.QUERY.clone();
+        REWRITE_QUERY_ENCODER.addSafeCharacter('%');
+    }
+
     /**
      * The rewrite rules that the valve will use.
      */
@@ -81,8 +97,7 @@ public class RewriteValve extends ValveBase {
 
 
     /**
-     * Relative path to the configuration file.
-     * Note: If the valve's container is a context, this will be relative to
+     * Relative path to the configuration file. Note: If the valve's container is a context, this will be relative to
      * /WEB-INF/.
      */
     protected String resourcePath = "rewrite.config";
@@ -102,7 +117,7 @@ public class RewriteValve extends ValveBase {
     /**
      * Maps to be used by the rules.
      */
-    protected Map<String, RewriteMap> maps = new ConcurrentHashMap<>();
+    protected Map<String,RewriteMap> maps = new ConcurrentHashMap<>();
 
 
     /**
@@ -143,8 +158,7 @@ public class RewriteValve extends ValveBase {
         if (getContainer() instanceof Context) {
             context = true;
             String webInfResourcePath = "/WEB-INF/" + resourcePath;
-            is = ((Context) getContainer()).getServletContext()
-                .getResourceAsStream(webInfResourcePath);
+            is = ((Context) getContainer()).getServletContext().getResourceAsStream(webInfResourcePath);
             if (containerLog.isDebugEnabled()) {
                 if (is == null) {
                     containerLog.debug(sm.getString("rewriteValve.noConfiguration", webInfResourcePath));
@@ -184,11 +198,15 @@ public class RewriteValve extends ValveBase {
 
     }
 
-    public void setConfiguration(String configuration)
-        throws Exception {
+    public void setConfiguration(String configuration) throws Exception {
         if (containerLog == null) {
             containerLog = LogFactory.getLog(getContainer().getLogName() + ".rewrite");
         }
+        for (RewriteMap map : maps.values()) {
+            if (map instanceof Lifecycle) {
+                ((Lifecycle) map).stop();
+            }
+        }
         maps.clear();
         parse(new BufferedReader(new StringReader(configuration)));
     }
@@ -198,7 +216,7 @@ public class RewriteValve extends ValveBase {
         for (String mapConfiguration : mapsConfiguration) {
             buffer.append(mapConfiguration).append("\r\n");
         }
-        if (mapsConfiguration.size() > 0) {
+        if (!mapsConfiguration.isEmpty()) {
             buffer.append("\r\n");
         }
         for (RewriteRule rule : rules) {
@@ -213,6 +231,7 @@ public class RewriteValve extends ValveBase {
     protected void parse(BufferedReader reader) throws LifecycleException {
         List<RewriteRule> rules = new ArrayList<>();
         List<RewriteCond> conditions = new ArrayList<>();
+        ArrayList<String> mapsConfiguration = new ArrayList<>();
         while (true) {
             try {
                 String line = reader.readLine();
@@ -223,8 +242,8 @@ public class RewriteValve extends ValveBase {
                 if (result instanceof RewriteRule) {
                     RewriteRule rule = (RewriteRule) result;
                     if (containerLog.isTraceEnabled()) {
-                        containerLog.trace("Add rule with pattern " + rule.getPatternString()
-                                + " and substitution " + rule.getSubstitutionString());
+                        containerLog.trace("Add rule with pattern " + rule.getPatternString() + " and substitution " +
+                                rule.getSubstitutionString());
                     }
                     for (int i = (conditions.size() - 1); i > 0; i--) {
                         if (conditions.get(i - 1).isOrnext()) {
@@ -233,12 +252,10 @@ public class RewriteValve extends ValveBase {
                     }
                     for (RewriteCond condition : conditions) {
                         if (containerLog.isTraceEnabled()) {
-                            RewriteCond cond = condition;
-                            containerLog.trace("Add condition " + cond.getCondPattern()
-                                    + " test " + cond.getTestString() + " to rule with pattern "
-                                    + rule.getPatternString() + " and substitution "
-                                    + rule.getSubstitutionString() + (cond.isOrnext() ? " [OR]" : "")
-                                    + (cond.isNocase() ? " [NC]" : ""));
+                            containerLog.trace("Add condition " + condition.getCondPattern() + " test " +
+                                condition.getTestString() + " to rule with pattern " + rule.getPatternString() +
+                                    " and substitution " + rule.getSubstitutionString() +
+                                    (condition.isOrnext() ? " [OR]" : "") + (condition.isNocase() ? " [NC]" : ""));
                         }
                         rule.addCondition(condition);
                     }
@@ -261,12 +278,14 @@ public class RewriteValve extends ValveBase {
                 containerLog.error(sm.getString("rewriteValve.readError"), e);
             }
         }
-        this.rules = rules.toArray(new RewriteRule[0]);
+        this.mapsConfiguration = mapsConfiguration;
 
         // Finish parsing the rules
-        for (RewriteRule rule : this.rules) {
+        for (RewriteRule rule : rules) {
             rule.parse(maps);
         }
+
+        this.rules = rules.toArray(new RewriteRule[0]);
     }
 
     @Override
@@ -283,8 +302,7 @@ public class RewriteValve extends ValveBase {
 
 
     @Override
-    public void invoke(Request request, Response response)
-        throws IOException, ServletException {
+    public void invoke(Request request, Response response) throws IOException, ServletException {
 
         if (!getEnabled() || rules == null || rules.length == 0) {
             getNext().invoke(request, response);
@@ -306,32 +324,59 @@ public class RewriteValve extends ValveBase {
 
             invoked.set(Boolean.TRUE);
 
-            // As long as MB isn't a char sequence or affiliated, this has to be
-            // converted to a string
+            // As long as MB isn't a char sequence or affiliated, this has to be converted to a string
             Charset uriCharset = request.getConnector().getURICharset();
             String originalQueryStringEncoded = request.getQueryString();
-            MessageBytes urlMB =
-                    context ? request.getRequestPathMB() : request.getDecodedRequestURIMB();
+            MessageBytes urlMB = context ? request.getRequestPathMB() : request.getDecodedRequestURIMB();
             urlMB.toChars();
             CharSequence urlDecoded = urlMB.getCharChunk();
+
+            /*
+             * The URL presented to the rewrite valve is the URL that is used for request mapping. That URL has been
+             * processed to: remove path parameters; remove the query string; decode; and normalize the URL. It may
+             * contain literal '%', '?' and/or ';' characters at this point.
+             *
+             * The re-write rules need to be able to process URLs with literal '?' characters and add query strings
+             * without the two becoming confused. The re-write rules also need to be able to insert literal '%'
+             * characters without them being confused with %nn encoding.
+             *
+             * To meet these requirement, the URL is processed as follows.
+             *
+             * Step 1. The URL is partially re-encoded by encodeForRewrite(). This method encodes any literal '%', ';'
+             * and/or '?' characters in the URL using the standard %nn form.
+             *
+             * Step 2. The re-write processing runs with the provided re-write rules against the partially encoded URL.
+             * If a re-write rule needs to insert a literal '%', ';' or '?', it must do so in %nn encoded form.
+             *
+             * Step 3. The URL (and query string if present) is re-encoded using the re-write specific encoders
+             * (REWRITE_DEFAULT_ENCODER and REWRITE_QUERY_ENCODER) that behave the same was as the standard encoders
+             * apart from '%' being treated as a safe character. This prevents double encoding of any '%' characters
+             * present in the URL from steps 1 or 2.
+             */
+
+            // Step 1. Encode URL for processing by the re-write rules.
+            CharSequence urlRewriteEncoded = encodeForRewrite(urlDecoded);
             CharSequence host = request.getServerName();
             boolean rewritten = false;
             boolean done = false;
             boolean qsa = false;
             boolean qsd = false;
+            boolean valveSkip = false;
+
+            // Step 2. Process the URL using the re-write rules.
             for (int i = 0; i < rules.length; i++) {
                 RewriteRule rule = rules[i];
-                CharSequence test = (rule.isHost()) ? host : urlDecoded;
+                CharSequence test = (rule.isHost()) ? host : urlRewriteEncoded;
                 CharSequence newtest = rule.evaluate(test, resolver);
-                if (newtest != null && !test.equals(newtest.toString())) {
+                if (newtest != null && !Objects.equals(test.toString(), newtest.toString())) {
                     if (containerLog.isTraceEnabled()) {
-                        containerLog.trace("Rewrote " + test + " as " + newtest
-                                + " with rule pattern " + rule.getPatternString());
+                        containerLog.trace(
+                                "Rewrote " + test + " as " + newtest + " with rule pattern " + rule.getPatternString());
                     }
                     if (rule.isHost()) {
                         host = newtest;
                     } else {
-                        urlDecoded = newtest;
+                        urlRewriteEncoded = newtest;
                     }
                     rewritten = true;
                 }
@@ -345,6 +390,10 @@ public class RewriteValve extends ValveBase {
                     qsd = true;
                 }
 
+                if (!valveSkip && newtest != null && rule.isValveSkip()) {
+                    valveSkip = true;
+                }
+
                 // Final reply
 
                 // - forbidden
@@ -364,29 +413,30 @@ public class RewriteValve extends ValveBase {
                 if (rule.isRedirect() && newtest != null) {
                     // Append the query string to the url if there is one and it
                     // hasn't been rewritten
-                    String urlStringDecoded = urlDecoded.toString();
-                    int index = urlStringDecoded.indexOf('?');
-                    String rewrittenQueryStringDecoded;
+                    String urlStringRewriteEncoded = urlRewriteEncoded.toString();
+                    int index = urlStringRewriteEncoded.indexOf('?');
+                    String rewrittenQueryStringRewriteEncoded;
                     if (index == -1) {
-                        rewrittenQueryStringDecoded = null;
+                        rewrittenQueryStringRewriteEncoded = null;
                     } else {
-                        rewrittenQueryStringDecoded = urlStringDecoded.substring(index + 1);
-                        urlStringDecoded = urlStringDecoded.substring(0, index);
+                        rewrittenQueryStringRewriteEncoded = urlStringRewriteEncoded.substring(index + 1);
+                        urlStringRewriteEncoded = urlStringRewriteEncoded.substring(0, index);
                     }
 
+                    // Step 3. Complete the 2nd stage to encoding.
                     StringBuilder urlStringEncoded =
-                            new StringBuilder(URLEncoder.DEFAULT.encode(urlStringDecoded, uriCharset));
-                    if (!qsd && originalQueryStringEncoded != null
-                            && originalQueryStringEncoded.length() > 0) {
-                        if (rewrittenQueryStringDecoded == null) {
+                            new StringBuilder(REWRITE_DEFAULT_ENCODER.encode(urlStringRewriteEncoded, uriCharset));
+
+                    if (!qsd && originalQueryStringEncoded != null && !originalQueryStringEncoded.isEmpty()) {
+                        if (rewrittenQueryStringRewriteEncoded == null) {
                             urlStringEncoded.append('?');
                             urlStringEncoded.append(originalQueryStringEncoded);
                         } else {
                             if (qsa) {
                                 // if qsa is specified append the query
                                 urlStringEncoded.append('?');
-                                urlStringEncoded.append(URLEncoder.QUERY.encode(
-                                        rewrittenQueryStringDecoded, uriCharset));
+                                urlStringEncoded.append(
+                                        REWRITE_QUERY_ENCODER.encode(rewrittenQueryStringRewriteEncoded, uriCharset));
                                 urlStringEncoded.append('&');
                                 urlStringEncoded.append(originalQueryStringEncoded);
                             } else if (index == urlStringEncoded.length() - 1) {
@@ -395,30 +445,30 @@ public class RewriteValve extends ValveBase {
                                 urlStringEncoded.deleteCharAt(index);
                             } else {
                                 urlStringEncoded.append('?');
-                                urlStringEncoded.append(URLEncoder.QUERY.encode(
-                                        rewrittenQueryStringDecoded, uriCharset));
+                                urlStringEncoded.append(
+                                        REWRITE_QUERY_ENCODER.encode(rewrittenQueryStringRewriteEncoded, uriCharset));
                             }
                         }
-                    } else if (rewrittenQueryStringDecoded != null) {
+                    } else if (rewrittenQueryStringRewriteEncoded != null) {
                         urlStringEncoded.append('?');
-                        urlStringEncoded.append(
-                                URLEncoder.QUERY.encode(rewrittenQueryStringDecoded, uriCharset));
+                        urlStringEncoded
+                                .append(REWRITE_QUERY_ENCODER.encode(rewrittenQueryStringRewriteEncoded, uriCharset));
                     }
 
                     // Insert the context if
                     // 1. this valve is associated with a context
                     // 2. the url starts with a leading slash
                     // 3. the url isn't absolute
-                    if (context && urlStringEncoded.charAt(0) == '/' &&
-                            !UriUtil.hasScheme(urlStringEncoded)) {
+                    if (context && urlStringEncoded.charAt(0) == '/' && !UriUtil.hasScheme(urlStringEncoded)) {
                         urlStringEncoded.insert(0, request.getContext().getEncodedPath());
                     }
+                    String redirectPath;
                     if (rule.isNoescape()) {
-                        response.sendRedirect(
-                                UDecoder.URLDecode(urlStringEncoded.toString(), uriCharset));
+                        redirectPath = UDecoder.URLDecode(urlStringEncoded.toString(), uriCharset);
                     } else {
-                        response.sendRedirect(urlStringEncoded.toString());
+                        redirectPath = urlStringEncoded.toString();
                     }
+                    response.sendRedirect(response.encodeRedirectURL(redirectPath));
                     response.setStatus(rule.getRedirectCode());
                     done = true;
                     break;
@@ -428,8 +478,7 @@ public class RewriteValve extends ValveBase {
 
                 // - cookie
                 if (rule.isCookie() && newtest != null) {
-                    Cookie cookie = new Cookie(rule.getCookieName(),
-                            rule.getCookieResult());
+                    Cookie cookie = new Cookie(rule.getCookieName(), rule.getCookieResult());
                     cookie.setDomain(rule.getCookieDomain());
                     cookie.setMaxAge(rule.getCookieLifetime());
                     cookie.setPath(rule.getCookiePath());
@@ -444,7 +493,7 @@ public class RewriteValve extends ValveBase {
                     }
                 }
                 // - content type (note: this will not force the content type, use a filter
-                //   to do that)
+                // to do that)
                 if (rule.isType() && newtest != null) {
                     response.setContentType(rule.getTypeValue());
                 }
@@ -480,13 +529,15 @@ public class RewriteValve extends ValveBase {
             if (rewritten) {
                 if (!done) {
                     // See if we need to replace the query string
-                    String urlStringDecoded = urlDecoded.toString();
-                    String queryStringDecoded = null;
-                    int queryIndex = urlStringDecoded.indexOf('?');
+                    String urlStringRewriteEncoded = urlRewriteEncoded.toString();
+                    String queryStringRewriteEncoded = null;
+                    int queryIndex = urlStringRewriteEncoded.indexOf('?');
                     if (queryIndex != -1) {
-                        queryStringDecoded = urlStringDecoded.substring(queryIndex+1);
-                        urlStringDecoded = urlStringDecoded.substring(0, queryIndex);
+                        queryStringRewriteEncoded = urlStringRewriteEncoded.substring(queryIndex + 1);
+                        urlStringRewriteEncoded = urlStringRewriteEncoded.substring(0, queryIndex);
                     }
+                    // Parse path parameters from rewrite production and populate request path parameters
+                    urlStringRewriteEncoded = org.apache.catalina.util.RequestUtil.stripPathParams(urlStringRewriteEncoded, request);
                     // Save the current context path before re-writing starts
                     String contextPath = null;
                     if (context) {
@@ -499,24 +550,25 @@ public class RewriteValve extends ValveBase {
                         // This is neither decoded nor normalized
                         chunk.append(contextPath);
                     }
-                    chunk.append(URLEncoder.DEFAULT.encode(urlStringDecoded, uriCharset));
+
+                    // Step 3. Complete the 2nd stage to encoding.
+                    chunk.append(REWRITE_DEFAULT_ENCODER.encode(urlStringRewriteEncoded, uriCharset));
                     // Decoded and normalized URI
                     // Rewriting may have denormalized the URL
-                    urlStringDecoded = RequestUtil.normalize(urlStringDecoded);
+                    urlStringRewriteEncoded = RequestUtil.normalize(urlStringRewriteEncoded);
                     request.getCoyoteRequest().decodedURI().setChars(MessageBytes.EMPTY_CHAR_ARRAY, 0, 0);
                     chunk = request.getCoyoteRequest().decodedURI().getCharChunk();
                     if (context) {
                         // This is decoded and normalized
                         chunk.append(request.getServletContext().getContextPath());
                     }
-                    chunk.append(urlStringDecoded);
+                    chunk.append(URLDecoder.decode(urlStringRewriteEncoded, uriCharset.name()));
                     // Set the new Query if there is one
-                    if (queryStringDecoded != null) {
+                    if (queryStringRewriteEncoded != null) {
                         request.getCoyoteRequest().queryString().setChars(MessageBytes.EMPTY_CHAR_ARRAY, 0, 0);
                         chunk = request.getCoyoteRequest().queryString().getCharChunk();
-                        chunk.append(URLEncoder.QUERY.encode(queryStringDecoded, uriCharset));
-                        if (qsa && originalQueryStringEncoded != null &&
-                                originalQueryStringEncoded.length() > 0) {
+                        chunk.append(REWRITE_QUERY_ENCODER.encode(queryStringRewriteEncoded, uriCharset));
+                        if (qsa && originalQueryStringEncoded != null && !originalQueryStringEncoded.isEmpty()) {
                             chunk.append('&');
                             chunk.append(originalQueryStringEncoded);
                         }
@@ -528,11 +580,12 @@ public class RewriteValve extends ValveBase {
                         chunk.append(host.toString());
                     }
                     request.getMappingData().recycle();
+                    request.recycleSessionInfo();
                     // Reinvoke the whole request recursively
                     Connector connector = request.getConnector();
                     try {
-                        if (!connector.getProtocolHandler().getAdapter().prepare(
-                                request.getCoyoteRequest(), response.getCoyoteResponse())) {
+                        if (!connector.getProtocolHandler().getAdapter().prepare(request.getCoyoteRequest(),
+                                response.getCoyoteResponse())) {
                             return;
                         }
                     } catch (Exception e) {
@@ -543,7 +596,15 @@ public class RewriteValve extends ValveBase {
                     pipeline.getFirst().invoke(request, response);
                 }
             } else {
-                getNext().invoke(request, response);
+                Valve next = getNext();
+                if (valveSkip) {
+                    next = next.getNext();
+                    if (next == null) {
+                        // Ignore and invoke the next valve normally
+                        next = getNext();
+                    }
+                }
+                next.invoke(request, response);
             }
 
         } finally {
@@ -555,11 +616,11 @@ public class RewriteValve extends ValveBase {
 
     /**
      * This factory method will parse a line formed like:
-     *
-     * Example:
-     *  RewriteCond %{REMOTE_HOST}  ^host1.*  [OR]
+     * <p>
+     * Example: {@code RewriteCond %{REMOTE_HOST} ^host1.* [OR]}
      *
      * @param line A line from the rewrite configuration
+     *
      * @return The condition, rule or map resulting from parsing the line
      */
     public static Object parse(String line) {
@@ -621,8 +682,8 @@ public class RewriteValve extends ValveBase {
                 } else if (rewriteMapClassName.startsWith("rnd:")) {
                     map = new RandomizedTextRewriteMap(rewriteMapClassName.substring("rnd:".length()), true);
                 } else if (rewriteMapClassName.startsWith("prg:")) {
-                    // FIXME: https://httpd.apache.org/docs/2.4/rewrite/rewritemap.html#prg
-                    // Likely not worth implementing further since this is a simpler CGI
+                    // https://httpd.apache.org/docs/2.4/rewrite/rewritemap.html#prg
+                    // Not worth implementing further since this is a simpler CGI
                     // piping stdin/stdout from an external native process
                     // Instead assume a class and use the RewriteMap interface
                     rewriteMapClassName = rewriteMapClassName.substring("prg:".length());
@@ -634,8 +695,7 @@ public class RewriteValve extends ValveBase {
                 }
                 if (map == null) {
                     try {
-                        map = (RewriteMap) (Class.forName(
-                                rewriteMapClassName).getConstructor().newInstance());
+                        map = (RewriteMap) (Class.forName(rewriteMapClassName).getConstructor().newInstance());
                     } catch (Exception e) {
                         throw new IllegalArgumentException(sm.getString("rewriteValve.invalidMapClassName", line));
                     }
@@ -664,9 +724,10 @@ public class RewriteValve extends ValveBase {
 
     /**
      * Parser for RewriteCond flags.
-     * @param line The configuration line being parsed
+     *
+     * @param line      The configuration line being parsed
      * @param condition The current condition
-     * @param flag The flag
+     * @param flag      The flag
      */
     protected static void parseCondFlag(String line, RewriteCond condition, String flag) {
         if (flag.equals("NC") || flag.equals("nocase")) {
@@ -681,6 +742,7 @@ public class RewriteValve extends ValveBase {
 
     /**
      * Parser for RewriteRule flags.
+     *
      * @param line The configuration line being parsed
      * @param rule The current rule
      * @param flag The flag
@@ -749,8 +811,8 @@ public class RewriteValve extends ValveBase {
             rule.setNoescape(true);
         } else if (flag.startsWith("next") || flag.startsWith("N")) {
             rule.setNext(true);
-        // Note: Proxy is not supported as Tomcat does not have proxy
-        //       capabilities
+            // Note: Proxy is not supported as Tomcat does not have proxy
+            // capabilities
         } else if (flag.startsWith("qsappend") || flag.startsWith("QSA")) {
             rule.setQsappend(true);
         } else if (flag.startsWith("qsdiscard") || flag.startsWith("QSD")) {
@@ -764,7 +826,7 @@ public class RewriteValve extends ValveBase {
                 } else if (flag.startsWith("R=")) {
                     flag = flag.substring("R=".length());
                 }
-                switch(flag) {
+                switch (flag) {
                     case "temp":
                         redirectCode = HttpServletResponse.SC_FOUND;
                         break;
@@ -795,8 +857,39 @@ public class RewriteValve extends ValveBase {
             }
             rule.setType(true);
             rule.setTypeValue(flag);
+        } else if (flag.startsWith("valveSkip") || flag.startsWith("VS")) {
+            rule.setValveSkip(true);
         } else {
             throw new IllegalArgumentException(sm.getString("rewriteValve.invalidFlags", line, flag));
         }
     }
+
+
+    private CharSequence encodeForRewrite(CharSequence input) {
+        StringBuilder result = null;
+        int pos = 0;
+        int mark = 0;
+        while (pos < input.length()) {
+            char c = input.charAt(pos);
+            if (c == '%' || c == ';' || c == '?') {
+                if (result == null) {
+                    result = new StringBuilder((int) (input.length() * 1.1));
+                }
+                result.append(input.subSequence(mark, pos));
+                result.append('%');
+                result.append(Character.forDigit((c >> 4) & 0xF, 16));
+                result.append(Character.forDigit(c & 0xF, 16));
+                mark = pos + 1;
+            }
+            pos++;
+        }
+        if (result != null) {
+            result.append(input.subSequence(mark, input.length()));
+            return result;
+        } else {
+            return input;
+        }
+    }
+
+
 }
