@@ -18,6 +18,7 @@ package org.apache.tomcat.websocket;
 
 import java.io.EOFException;
 import java.io.IOException;
+import java.net.SocketAddress;
 import java.nio.ByteBuffer;
 import java.nio.channels.AsynchronousSocketChannel;
 import java.nio.channels.CompletionHandler;
@@ -43,15 +44,13 @@ import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.res.StringManager;
 
 /**
- * Wraps the {@link AsynchronousSocketChannel} with SSL/TLS. This needs a lot
- * more testing before it can be considered robust.
+ * Wraps the {@link AsynchronousSocketChannel} with SSL/TLS. This needs a lot more testing before it can be considered
+ * robust.
  */
 public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
 
-    private final Log log =
-            LogFactory.getLog(AsyncChannelWrapperSecure.class);
-    private static final StringManager sm =
-            StringManager.getManager(Constants.PACKAGE_NAME);
+    private final Log log = LogFactory.getLog(AsyncChannelWrapperSecure.class);
+    private static final StringManager sm = StringManager.getManager(AsyncChannelWrapperSecure.class);
 
     private static final ByteBuffer DUMMY = ByteBuffer.allocate(16921);
     private final AsynchronousSocketChannel socketChannel;
@@ -59,13 +58,11 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
     private final ByteBuffer socketReadBuffer;
     private final ByteBuffer socketWriteBuffer;
     // One thread for read, one for write
-    private final ExecutorService executor =
-            Executors.newFixedThreadPool(2, new SecureIOThreadFactory());
-    private AtomicBoolean writing = new AtomicBoolean(false);
-    private AtomicBoolean reading = new AtomicBoolean(false);
+    private final ExecutorService executor = Executors.newFixedThreadPool(2, new SecureIOThreadFactory());
+    private final AtomicBoolean writing = new AtomicBoolean(false);
+    private final AtomicBoolean reading = new AtomicBoolean(false);
 
-    public AsyncChannelWrapperSecure(AsynchronousSocketChannel socketChannel,
-            SSLEngine sslEngine) {
+    public AsyncChannelWrapperSecure(AsynchronousSocketChannel socketChannel, SSLEngine sslEngine) {
         this.socketChannel = socketChannel;
         this.sslEngine = sslEngine;
 
@@ -76,11 +73,10 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
 
     @Override
     public Future<Integer> read(ByteBuffer dst) {
-        WrapperFuture<Integer,Void> future = new WrapperFuture<>();
+        WrapperFuture<Integer, Void> future = new WrapperFuture<>();
 
         if (!reading.compareAndSet(false, true)) {
-            throw new IllegalStateException(sm.getString(
-                    "asyncChannelWrapperSecure.concurrentRead"));
+            throw new IllegalStateException(sm.getString("asyncChannelWrapperSecure.concurrentRead"));
         }
 
         ReadTask readTask = new ReadTask(dst, future);
@@ -91,15 +87,12 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
     }
 
     @Override
-    public <B,A extends B> void read(ByteBuffer dst, A attachment,
-            CompletionHandler<Integer,B> handler) {
+    public <B, A extends B> void read(ByteBuffer dst, A attachment, CompletionHandler<Integer, B> handler) {
 
-        WrapperFuture<Integer,B> future =
-                new WrapperFuture<>(handler, attachment);
+        WrapperFuture<Integer, B> future = new WrapperFuture<>(handler, attachment);
 
         if (!reading.compareAndSet(false, true)) {
-            throw new IllegalStateException(sm.getString(
-                    "asyncChannelWrapperSecure.concurrentRead"));
+            throw new IllegalStateException(sm.getString("asyncChannelWrapperSecure.concurrentRead"));
         }
 
         ReadTask readTask = new ReadTask(dst, future);
@@ -110,33 +103,27 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
     @Override
     public Future<Integer> write(ByteBuffer src) {
 
-        WrapperFuture<Long,Void> inner = new WrapperFuture<>();
+        WrapperFuture<Long, Void> inner = new WrapperFuture<>();
 
         if (!writing.compareAndSet(false, true)) {
-            throw new IllegalStateException(sm.getString(
-                    "asyncChannelWrapperSecure.concurrentWrite"));
+            throw new IllegalStateException(sm.getString("asyncChannelWrapperSecure.concurrentWrite"));
         }
 
-        WriteTask writeTask =
-                new WriteTask(new ByteBuffer[] {src}, 0, 1, inner);
+        WriteTask writeTask = new WriteTask(new ByteBuffer[] { src }, 0, 1, inner);
 
         executor.execute(writeTask);
 
-        Future<Integer> future = new LongToIntegerFuture(inner);
-        return future;
+        return new LongToIntegerFuture(inner);
     }
 
     @Override
-    public <B,A extends B> void write(ByteBuffer[] srcs, int offset, int length,
-            long timeout, TimeUnit unit, A attachment,
-            CompletionHandler<Long,B> handler) {
+    public <B, A extends B> void write(ByteBuffer[] srcs, int offset, int length, long timeout, TimeUnit unit,
+            A attachment, CompletionHandler<Long, B> handler) {
 
-        WrapperFuture<Long,B> future =
-                new WrapperFuture<>(handler, attachment);
+        WrapperFuture<Long, B> future = new WrapperFuture<>(handler, attachment);
 
         if (!writing.compareAndSet(false, true)) {
-            throw new IllegalStateException(sm.getString(
-                    "asyncChannelWrapperSecure.concurrentWrite"));
+            throw new IllegalStateException(sm.getString("asyncChannelWrapperSecure.concurrentWrite"));
         }
 
         WriteTask writeTask = new WriteTask(srcs, offset, length, future);
@@ -157,7 +144,7 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
     @Override
     public Future<Void> handshake() throws SSLException {
 
-        WrapperFuture<Void,Void> wFuture = new WrapperFuture<>();
+        WrapperFuture<Void, Void> wFuture = new WrapperFuture<>();
 
         Thread t = new WebSocketSslHandshakeThread(wFuture);
         t.start();
@@ -166,15 +153,20 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
     }
 
 
+    @Override
+    public SocketAddress getLocalAddress() throws IOException {
+        return socketChannel.getLocalAddress();
+    }
+
+
     private class WriteTask implements Runnable {
 
         private final ByteBuffer[] srcs;
         private final int offset;
         private final int length;
-        private final WrapperFuture<Long,?> future;
+        private final WrapperFuture<Long, ?> future;
 
-        public WriteTask(ByteBuffer[] srcs, int offset, int length,
-                WrapperFuture<Long,?> future) {
+        WriteTask(ByteBuffer[] srcs, int offset, int length, WrapperFuture<Long, ?> future) {
             this.srcs = srcs;
             this.future = future;
             this.offset = offset;
@@ -202,8 +194,7 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
                         } else {
                             // Status.BUFFER_UNDERFLOW - only happens on unwrap
                             // Status.CLOSED - unexpected
-                            throw new IllegalStateException(sm.getString(
-                                    "asyncChannelWrapperSecure.statusWrap"));
+                            throw new IllegalStateException(sm.getString("asyncChannelWrapperSecure.statusWrap"));
                         }
 
                         // Check for tasks
@@ -220,8 +211,7 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
                         // Do the write
                         int toWrite = r.bytesProduced();
                         while (toWrite > 0) {
-                            Future<Integer> f =
-                                    socketChannel.write(socketWriteBuffer);
+                            Future<Integer> f = socketChannel.write(socketWriteBuffer);
                             Integer socketWrite = f.get();
                             toWrite -= socketWrite.intValue();
                         }
@@ -232,8 +222,7 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
                 if (writing.compareAndSet(true, false)) {
                     future.complete(Long.valueOf(written));
                 } else {
-                    future.fail(new IllegalStateException(sm.getString(
-                            "asyncChannelWrapperSecure.wrongStateWrite")));
+                    future.fail(new IllegalStateException(sm.getString("asyncChannelWrapperSecure.wrongStateWrite")));
                 }
             } catch (Exception e) {
                 writing.set(false);
@@ -246,9 +235,9 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
     private class ReadTask implements Runnable {
 
         private final ByteBuffer dest;
-        private final WrapperFuture<Integer,?> future;
+        private final WrapperFuture<Integer, ?> future;
 
-        public ReadTask(ByteBuffer dest, WrapperFuture<Integer,?> future) {
+        ReadTask(ByteBuffer dest, WrapperFuture<Integer, ?> future) {
             this.dest = dest;
             this.future = future;
         }
@@ -268,8 +257,7 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
                         Future<Integer> f = socketChannel.read(socketReadBuffer);
                         Integer socketRead = f.get();
                         if (socketRead.intValue() == -1) {
-                            throw new EOFException(sm.getString(
-                                    "asyncChannelWrapperSecure.eof"));
+                            throw new EOFException(sm.getString("asyncChannelWrapperSecure.eof"));
                         }
                     }
 
@@ -277,8 +265,7 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
 
                     if (socketReadBuffer.hasRemaining()) {
                         // Decrypt the data in the buffer
-                        SSLEngineResult r =
-                                sslEngine.unwrap(socketReadBuffer, dest);
+                        SSLEngineResult r = sslEngine.unwrap(socketReadBuffer, dest);
                         read += r.bytesProduced();
                         Status s = r.getStatus();
 
@@ -303,16 +290,15 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
                             // buffer size required but an explicit exception is
                             // clearer.
                             if (reading.compareAndSet(true, false)) {
-                                throw new ReadBufferOverflowException(sslEngine.
-                                        getSession().getApplicationBufferSize());
+                                throw new ReadBufferOverflowException(
+                                        sslEngine.getSession().getApplicationBufferSize());
                             } else {
-                                future.fail(new IllegalStateException(sm.getString(
-                                        "asyncChannelWrapperSecure.wrongStateRead")));
+                                future.fail(new IllegalStateException(
+                                        sm.getString("asyncChannelWrapperSecure.wrongStateRead")));
                             }
                         } else {
                             // Status.CLOSED - unexpected
-                            throw new IllegalStateException(sm.getString(
-                                    "asyncChannelWrapperSecure.statusUnwrap"));
+                            throw new IllegalStateException(sm.getString("asyncChannelWrapperSecure.statusUnwrap"));
                         }
 
                         // Check for tasks
@@ -332,10 +318,10 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
                 if (reading.compareAndSet(true, false)) {
                     future.complete(Integer.valueOf(read));
                 } else {
-                    future.fail(new IllegalStateException(sm.getString(
-                            "asyncChannelWrapperSecure.wrongStateRead")));
+                    future.fail(new IllegalStateException(sm.getString("asyncChannelWrapperSecure.wrongStateRead")));
                 }
-            } catch (Exception e) {
+            } catch (RuntimeException | ReadBufferOverflowException | SSLException | EOFException | ExecutionException
+                    | InterruptedException e) {
                 reading.set(false);
                 future.fail(e);
             }
@@ -345,12 +331,12 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
 
     private class WebSocketSslHandshakeThread extends Thread {
 
-        private final WrapperFuture<Void,Void> hFuture;
+        private final WrapperFuture<Void, Void> hFuture;
 
         private HandshakeStatus handshakeStatus;
         private Status resultStatus;
 
-        public WebSocketSslHandshakeThread(WrapperFuture<Void,Void> hFuture) {
+        WebSocketSslHandshakeThread(WrapperFuture<Void, Void> hFuture) {
             this.hFuture = hFuture;
         }
 
@@ -366,35 +352,30 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
 
                 boolean handshaking = true;
 
-                while(handshaking) {
+                while (handshaking) {
                     switch (handshakeStatus) {
                         case NEED_WRAP: {
                             socketWriteBuffer.clear();
-                            SSLEngineResult r =
-                                    sslEngine.wrap(DUMMY, socketWriteBuffer);
+                            SSLEngineResult r = sslEngine.wrap(DUMMY, socketWriteBuffer);
                             checkResult(r, true);
                             socketWriteBuffer.flip();
-                            Future<Integer> fWrite =
-                                    socketChannel.write(socketWriteBuffer);
+                            Future<Integer> fWrite = socketChannel.write(socketWriteBuffer);
                             fWrite.get();
                             break;
                         }
                         case NEED_UNWRAP: {
                             socketReadBuffer.compact();
-                            if (socketReadBuffer.position() == 0 ||
-                                    resultStatus == Status.BUFFER_UNDERFLOW) {
-                                Future<Integer> fRead =
-                                        socketChannel.read(socketReadBuffer);
+                            if (socketReadBuffer.position() == 0 || resultStatus == Status.BUFFER_UNDERFLOW) {
+                                Future<Integer> fRead = socketChannel.read(socketReadBuffer);
                                 fRead.get();
                             }
                             socketReadBuffer.flip();
-                            SSLEngineResult r =
-                                    sslEngine.unwrap(socketReadBuffer, DUMMY);
+                            SSLEngineResult r = sslEngine.unwrap(socketReadBuffer, DUMMY);
                             checkResult(r, false);
                             break;
                         }
                         case NEED_TASK: {
-                            Runnable r = null;
+                            Runnable r;
                             while ((r = sslEngine.getDelegatedTask()) != null) {
                                 r.run();
                             }
@@ -405,53 +386,52 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
                             handshaking = false;
                             break;
                         }
-                        default: {
-                            throw new SSLException("TODO");
+                        case NOT_HANDSHAKING: {
+                            // Don't expect to see this during a handshake
+                            throw new SSLException(sm.getString("asyncChannelWrapperSecure.notHandshaking"));
                         }
                     }
                 }
-            } catch (SSLException | InterruptedException |
-                    ExecutionException e) {
+            } catch (Exception e) {
                 hFuture.fail(e);
+                return;
             }
 
             hFuture.complete(null);
         }
 
-        private void checkResult(SSLEngineResult result, boolean wrap)
-                throws SSLException {
+        private void checkResult(SSLEngineResult result, boolean wrap) throws SSLException {
 
             handshakeStatus = result.getHandshakeStatus();
             resultStatus = result.getStatus();
 
-            if (resultStatus != Status.OK &&
-                    (wrap || resultStatus != Status.BUFFER_UNDERFLOW)) {
-                throw new SSLException("TODO");
+            if (resultStatus != Status.OK && (wrap || resultStatus != Status.BUFFER_UNDERFLOW)) {
+                throw new SSLException(sm.getString("asyncChannelWrapperSecure.check.notOk", resultStatus));
             }
             if (wrap && result.bytesConsumed() != 0) {
-                throw new SSLException("TODO");
+                throw new SSLException(sm.getString("asyncChannelWrapperSecure.check.wrap"));
             }
             if (!wrap && result.bytesProduced() != 0) {
-                throw new SSLException("TODO");
+                throw new SSLException(sm.getString("asyncChannelWrapperSecure.check.unwrap"));
             }
         }
     }
 
 
-    private static class WrapperFuture<T,A> implements Future<T> {
+    private static class WrapperFuture<T, A> implements Future<T> {
 
-        private final CompletionHandler<T,A> handler;
+        private final CompletionHandler<T, A> handler;
         private final A attachment;
 
         private volatile T result = null;
         private volatile Throwable throwable = null;
-        private CountDownLatch completionLatch = new CountDownLatch(1);
+        private final CountDownLatch completionLatch = new CountDownLatch(1);
 
-        public WrapperFuture() {
+        WrapperFuture() {
             this(null, null);
         }
 
-        public WrapperFuture(CompletionHandler<T,A> handler, A attachment) {
+        WrapperFuture(CompletionHandler<T, A> handler, A attachment) {
             this.handler = handler;
             this.attachment = attachment;
         }
@@ -499,11 +479,9 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
         }
 
         @Override
-        public T get(long timeout, TimeUnit unit)
-                throws InterruptedException, ExecutionException,
-                TimeoutException {
+        public T get(long timeout, TimeUnit unit) throws InterruptedException, ExecutionException, TimeoutException {
             boolean latchResult = completionLatch.await(timeout, unit);
-            if (latchResult == false) {
+            if (!latchResult) {
                 throw new TimeoutException();
             }
             if (throwable != null) {
@@ -517,7 +495,7 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
 
         private final Future<Long> wrapped;
 
-        public LongToIntegerFuture(Future<Long> wrapped) {
+        LongToIntegerFuture(Future<Long> wrapped) {
             this.wrapped = wrapped;
         }
 
@@ -540,20 +518,17 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
         public Integer get() throws InterruptedException, ExecutionException {
             Long result = wrapped.get();
             if (result.longValue() > Integer.MAX_VALUE) {
-                throw new ExecutionException(sm.getString(
-                        "asyncChannelWrapperSecure.tooBig", result), null);
+                throw new ExecutionException(sm.getString("asyncChannelWrapperSecure.tooBig", result), null);
             }
             return Integer.valueOf(result.intValue());
         }
 
         @Override
         public Integer get(long timeout, TimeUnit unit)
-                throws InterruptedException, ExecutionException,
-                TimeoutException {
+                throws InterruptedException, ExecutionException, TimeoutException {
             Long result = wrapped.get(timeout, unit);
             if (result.longValue() > Integer.MAX_VALUE) {
-                throw new ExecutionException(sm.getString(
-                        "asyncChannelWrapperSecure.tooBig", result), null);
+                throw new ExecutionException(sm.getString("asyncChannelWrapperSecure.tooBig", result), null);
             }
             return Integer.valueOf(result.intValue());
         }
@@ -562,7 +537,7 @@ public class AsyncChannelWrapperSecure implements AsyncChannelWrapper {
 
     private static class SecureIOThreadFactory implements ThreadFactory {
 
-        private AtomicInteger count = new AtomicInteger(0);
+        private final AtomicInteger count = new AtomicInteger(0);
 
         @Override
         public Thread newThread(Runnable r) {
