@@ -44,6 +44,13 @@ public class PerMessageDeflate implements Transformation {
 
     public static final String NAME = "permessage-deflate";
 
+    public static final TransformationBuilder BUILDER = new TransformationBuilder() {
+        @Override
+        public Transformation build(List<List<Parameter>> preferences, boolean isServer) {
+            return PerMessageDeflate.build(preferences, isServer);
+        }
+    };
+
     private final boolean serverContextTakeover;
     private final int serverMaxWindowBits;
     private final boolean clientContextTakeover;
@@ -61,7 +68,7 @@ public class PerMessageDeflate implements Transformation {
     // Flag to track if a message is completely empty
     private volatile boolean emptyMessage = true;
 
-    static PerMessageDeflate negotiate(List<List<Parameter>> preferences, boolean isServer) {
+    static PerMessageDeflate build(List<List<Parameter>> preferences, boolean isServer) {
         // Accept the first preference that the endpoint is able to support
         for (List<Parameter> preference : preferences) {
             boolean ok = true;
@@ -314,13 +321,17 @@ public class PerMessageDeflate implements Transformation {
 
         for (MessagePart uncompressedPart : uncompressedParts) {
             byte opCode = uncompressedPart.getOpCode();
-            boolean emptyPart = uncompressedPart.getPayload().limit() == 0;
-            emptyMessage = emptyMessage && emptyPart;
             if (Util.isControl(opCode)) {
                 // Control messages can appear in the middle of other messages
-                // and must not be compressed. Pass it straight through
+                // and must not be compressed. Pass it straight through.
                 allCompressedParts.add(uncompressedPart);
-            } else if (emptyMessage && uncompressedPart.isFin()) {
+                continue;
+            }
+
+            if (uncompressedPart.getPayload().limit() != 0) {
+                emptyMessage = false;
+            }
+            if (emptyMessage && uncompressedPart.isFin()) {
                 // Zero length messages can't be compressed so pass the
                 // final (empty) part straight through.
                 allCompressedParts.add(uncompressedPart);
@@ -329,9 +340,15 @@ public class PerMessageDeflate implements Transformation {
                 ByteBuffer uncompressedPayload = uncompressedPart.getPayload();
                 SendHandler uncompressedIntermediateHandler = uncompressedPart.getIntermediateHandler();
 
-                deflater.setInput(uncompressedPayload.array(),
-                        uncompressedPayload.arrayOffset() + uncompressedPayload.position(),
-                        uncompressedPayload.remaining());
+                if (uncompressedPayload.hasArray()) {
+                    deflater.setInput(uncompressedPayload.array(),
+                            uncompressedPayload.arrayOffset() + uncompressedPayload.position(),
+                            uncompressedPayload.remaining());
+                } else {
+                    byte[] bytes = new byte[uncompressedPayload.remaining()];
+                    uncompressedPayload.get(bytes);
+                    deflater.setInput(bytes, 0, bytes.length);
+                }
 
                 int flush = (uncompressedPart.isFin() ? Deflater.SYNC_FLUSH : Deflater.NO_FLUSH);
                 boolean deflateRequired = true;
@@ -384,14 +401,14 @@ public class PerMessageDeflate implements Transformation {
                         compressedPart = new MessagePart(false, getRsv(uncompressedPart), opCode, compressedPayload,
                                 uncompressedIntermediateHandler, uncompressedIntermediateHandler,
                                 blockingWriteTimeoutExpiry);
-                    } else if (!fin && full && needsInput) {
+                    } else if (!fin && full/* note: needsInput is true here*/) {
                         // Write buffer full and input message not fully read.
                         // Output and get more data.
                         compressedPart = new MessagePart(false, getRsv(uncompressedPart), opCode, compressedPayload,
                                 uncompressedIntermediateHandler, uncompressedIntermediateHandler,
                                 blockingWriteTimeoutExpiry);
                         deflateRequired = false;
-                    } else if (fin && full && needsInput) {
+                    } else if (fin && full/* note: needsInput is true here*/) {
                         // Write buffer full. Input fully read. Deflater may be
                         // in one of four states:
                         // - output complete (just happened to align with end of
