@@ -24,10 +24,16 @@ import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
 import java.net.HttpURLConnection;
 import java.net.URL;
+import java.nio.charset.Charset;
+import java.nio.charset.StandardCharsets;
+import java.nio.charset.UnsupportedCharsetException;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Enumeration;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Locale;
+import java.util.Map;
 import java.util.TreeMap;
 
 import javax.servlet.ServletException;
@@ -36,11 +42,10 @@ import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 
 import org.junit.Assert;
-import org.junit.BeforeClass;
-import org.junit.Ignore;
 import org.junit.Test;
 
 import org.apache.catalina.Context;
+import org.apache.catalina.Globals;
 import org.apache.catalina.authenticator.BasicAuthenticator;
 import org.apache.catalina.filters.FailedRequestFilter;
 import org.apache.catalina.startup.SimpleHttpClient;
@@ -49,6 +54,8 @@ import org.apache.catalina.startup.Tomcat;
 import org.apache.catalina.startup.TomcatBaseTest;
 import org.apache.tomcat.unittest.TesterRequest;
 import org.apache.tomcat.util.buf.ByteChunk;
+import org.apache.tomcat.util.buf.EncodedSolidusHandling;
+import org.apache.tomcat.util.buf.StringUtils;
 import org.apache.tomcat.util.descriptor.web.FilterDef;
 import org.apache.tomcat.util.descriptor.web.FilterMap;
 import org.apache.tomcat.util.descriptor.web.LoginConfig;
@@ -58,12 +65,6 @@ import org.apache.tomcat.util.descriptor.web.LoginConfig;
  */
 public class TestRequest extends TomcatBaseTest {
 
-    @BeforeClass
-    public static void setup() {
-        // Some of these tests need this and it used statically so set it once
-        System.setProperty("org.apache.tomcat.util.buf.UDecoder.ALLOW_ENCODED_SLASH", "true");
-    }
-
     /**
      * Test case for https://bz.apache.org/bugzilla/show_bug.cgi?id=37794
      * POST parameters are not returned from a call to
@@ -85,8 +86,6 @@ public class TestRequest extends TomcatBaseTest {
         client.doRequest(1, false); // 1 byte - too small should fail
         Assert.assertTrue(client.isResponse413());
 
-        client.reset();
-
         // Edge cases around actual content length
         client.reset();
         client.doRequest(6, false); // Too small should fail
@@ -162,12 +161,14 @@ public class TestRequest extends TomcatBaseTest {
 
         private boolean init;
 
-        public Bug37794Client(boolean createFilter) {
+        Bug37794Client(boolean createFilter) {
             this.createFilter = createFilter;
         }
 
         private synchronized void init() throws Exception {
-            if (init) return;
+            if (init) {
+              return;
+            }
 
             Tomcat tomcat = getTomcatInstance();
             Context root = tomcat.addContext("", TEMP_DIR);
@@ -207,9 +208,9 @@ public class TestRequest extends TomcatBaseTest {
                 String[] request = new String[2];
                 if (ucChunkedHead) {
                     request[0] =
-                        "POST /test HTTP/1.1" + CRLF +
+                        "POST http://localhost:8080/test HTTP/1.1" + CRLF +
                         "Host: localhost:8080" + CRLF +
-                        "content-type: application/x-www-form-urlencoded" + CRLF +
+                        SimpleHttpClient.HTTP_HEADER_CONTENT_TYPE_FORM_URL_ENCODING +
                         "Transfer-Encoding: CHUNKED" + CRLF +
                         "Connection: close" + CRLF +
                         CRLF +
@@ -217,9 +218,9 @@ public class TestRequest extends TomcatBaseTest {
                         "a=1" + CRLF;
                 } else {
                     request[0] =
-                        "POST /test HTTP/1.1" + CRLF +
+                        "POST http://localhost:8080/test HTTP/1.1" + CRLF +
                         "Host: localhost:8080" + CRLF +
-                        "content-type: application/x-www-form-urlencoded" + CRLF +
+                        SimpleHttpClient.HTTP_HEADER_CONTENT_TYPE_FORM_URL_ENCODING +
                         "Transfer-Encoding: chunked" + CRLF +
                         "Connection: close" + CRLF +
                         CRLF +
@@ -270,7 +271,7 @@ public class TestRequest extends TomcatBaseTest {
         Tomcat tomcat = getTomcatInstance();
 
         // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
+        Context ctx = getProgrammaticRootContext();
 
         // Add the Servlet
         Tomcat.addServlet(ctx, "servlet", new EchoQueryStringServlet());
@@ -314,7 +315,7 @@ public class TestRequest extends TomcatBaseTest {
         Tomcat tomcat = getTomcatInstance();
 
         // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
+        Context ctx = getProgrammaticRootContext();
 
         LoginConfig config = new LoginConfig();
         config.setAuthMethod("BASIC");
@@ -346,17 +347,21 @@ public class TestRequest extends TomcatBaseTest {
 
             req.login(USER, PWD);
 
-            if (!req.getRemoteUser().equals(USER))
-                throw new ServletException();
-            if (!req.getUserPrincipal().getName().equals(USER))
-                throw new ServletException();
+            if (!req.getRemoteUser().equals(USER)) {
+              throw new ServletException();
+            }
+            if (!req.getUserPrincipal().getName().equals(USER)) {
+              throw new ServletException();
+            }
 
             req.logout();
 
-            if (req.getRemoteUser() != null)
-                throw new ServletException();
-            if (req.getUserPrincipal() != null)
-                throw new ServletException();
+            if (req.getRemoteUser() != null) {
+              throw new ServletException();
+            }
+            if (req.getUserPrincipal() != null) {
+              throw new ServletException();
+            }
 
             resp.getWriter().write(OK);
         }
@@ -417,7 +422,7 @@ public class TestRequest extends TomcatBaseTest {
         // Make sure POST works properly
         //
         // POST with separate GET and POST parameters
-        client.doRequest("POST", "foo=bar", "application/x-www-form-urlencoded", "bar=baz", true);
+        client.doRequest("POST", "foo=bar", Globals.CONTENT_TYPE_FORM_URL_ENCODING, "bar=baz", true);
 
         Assert.assertTrue("Non-200 response for POST request",
                    client.isResponse200());
@@ -428,7 +433,7 @@ public class TestRequest extends TomcatBaseTest {
         client.reset();
 
         // POST with overlapping GET and POST parameters
-        client.doRequest("POST", "foo=bar&bar=foo", "application/x-www-form-urlencoded", "bar=baz&foo=baz", true);
+        client.doRequest("POST", "foo=bar&bar=foo", Globals.CONTENT_TYPE_FORM_URL_ENCODING, "bar=baz&foo=baz", true);
 
         Assert.assertTrue("Non-200 response for POST request",
                    client.isResponse200());
@@ -439,7 +444,7 @@ public class TestRequest extends TomcatBaseTest {
         client.reset();
 
         // PUT without POST-style parsing
-        client.doRequest("PUT", "foo=bar&bar=foo", "application/x-www-form-urlencoded", "bar=baz&foo=baz", false);
+        client.doRequest("PUT", "foo=bar&bar=foo", Globals.CONTENT_TYPE_FORM_URL_ENCODING, "bar=baz&foo=baz", false);
 
         Assert.assertTrue("Non-200 response for PUT/noparse request",
                    client.isResponse200());
@@ -450,7 +455,7 @@ public class TestRequest extends TomcatBaseTest {
         client.reset();
 
         // PUT with POST-style parsing
-        client.doRequest("PUT", "foo=bar&bar=foo", "application/x-www-form-urlencoded", "bar=baz&foo=baz", true);
+        client.doRequest("PUT", "foo=bar&bar=foo", Globals.CONTENT_TYPE_FORM_URL_ENCODING, "bar=baz&foo=baz", true);
 
         Assert.assertTrue("Non-200 response for PUT request",
                    client.isResponse200());
@@ -459,14 +464,6 @@ public class TestRequest extends TomcatBaseTest {
                      client.getResponseBody());
 
         client.reset();
-
-        /*
-        private Exception doRequest(String method,
-                                    String queryString,
-                                    String contentType,
-                                    String requestBody,
-                                    boolean allowBody) {
-        */
     }
 
     @Test
@@ -525,16 +522,16 @@ public class TestRequest extends TomcatBaseTest {
             for(String name: parameters.keySet()) {
                 String[] values = req.getParameterValues(name);
 
-                java.util.Arrays.sort(values);
+                Arrays.sort(values);
 
-                for(int i=0; i<values.length; ++i)
-                {
-                    if(first)
-                        first = false;
-                    else
-                        out.print(",");
+                for (String value : values) {
+                    if (first) {
+                      first = false;
+                    } else {
+                      out.print(",");
+                    }
 
-                    out.print(name + "=" + values[i]);
+                    out.print(name + "=" + value);
                 }
             }
         }
@@ -548,7 +545,9 @@ public class TestRequest extends TomcatBaseTest {
         private boolean init;
 
         private synchronized void init() throws Exception {
-            if (init) return;
+            if (init) {
+              return;
+            }
 
             Tomcat tomcat = getTomcatInstance();
             Context root = tomcat.addContext("", TEMP_DIR);
@@ -570,17 +569,20 @@ public class TestRequest extends TomcatBaseTest {
 
             try {
                 init();
-                if(allowBody)
-                    tomcat.getConnector().setParseBodyMethods(method);
-                else
-                    tomcat.getConnector().setParseBodyMethods(""); // never parse
+                if(allowBody) {
+                  tomcat.getConnector().setParseBodyMethods(method);
+                }
+                else {
+                  tomcat.getConnector().setParseBodyMethods(""); // never parse
+                }
 
                 // Open connection
                 connect();
 
                 // Re-encode the request body so that bytes = characters
-                if(null != requestBody)
-                    requestBody = new String(requestBody.getBytes("UTF-8"), "ASCII");
+                if(null != requestBody) {
+                  requestBody = new String(requestBody.getBytes("UTF-8"), "ASCII");
+                }
 
                 // Send specified request body using method
                 String[] request = {
@@ -588,7 +590,7 @@ public class TestRequest extends TomcatBaseTest {
                      method + " http://localhost:" + getPort() + "/echo"
                      + (null == queryString ? "" : ("?" + queryString))
                      + " HTTP/1.1" + CRLF
-                     + "Host: localhost" + CRLF
+                     + "Host: localhost:" + getPort() + CRLF
                      + (null == contentType ? ""
                         : ("Content-Type: " + contentType + CRLF))
                      + "Connection: close" + CRLF
@@ -798,12 +800,12 @@ public class TestRequest extends TomcatBaseTest {
 
     @Test
     public void testBug57215c() throws Exception {
-        doBug56501("/path", "/%2Fpath", "/%2Fpath");
+        doBug56501("/path", "/%2Fpath", "/%2Fpath", EncodedSolidusHandling.DECODE);
     }
 
     @Test
     public void testBug57215d() throws Exception {
-        doBug56501("/path", "/%2Fpath%2F", "/%2Fpath");
+        doBug56501("/path", "/%2Fpath%2F", "/%2Fpath", EncodedSolidusHandling.DECODE);
     }
 
     @Test
@@ -813,15 +815,22 @@ public class TestRequest extends TomcatBaseTest {
 
     @Test
     public void testBug57215f() throws Exception {
-        doBug56501("/path", "/foo/..%2fpath", "/foo/..%2fpath");
+        doBug56501("/path", "/foo/..%2fpath", "/foo/..%2fpath", EncodedSolidusHandling.DECODE);
     }
 
-    private void doBug56501(String deployPath, String requestPath, String expected)
-            throws Exception {
+    private void doBug56501(String deployPath, String requestPath, String expected) throws Exception {
+        doBug56501(deployPath, requestPath, expected, EncodedSolidusHandling.REJECT);
+    }
+
+
+    private void doBug56501(String deployPath, String requestPath, String expected,
+            EncodedSolidusHandling encodedSolidusHandling) throws Exception {
 
         // Setup Tomcat instance
         Tomcat tomcat = getTomcatInstance();
 
+        tomcat.getConnector().setEncodedSolidusHandling(encodedSolidusHandling.getValue());
+
         // No file system docBase required
         Context ctx = tomcat.addContext(deployPath, null);
         ctx.setAllowMultipleLeadingForwardSlashInPath(true);
@@ -839,7 +848,7 @@ public class TestRequest extends TomcatBaseTest {
         Assert.assertEquals(expected, resultPath);
     }
 
-    private class Bug56501Servlet extends HttpServlet {
+    private static class Bug56501Servlet extends HttpServlet {
 
         private static final long serialVersionUID = 1L;
 
@@ -883,22 +892,129 @@ public class TestRequest extends TomcatBaseTest {
 
 
     @Test
-    @Ignore("Used to check performance of different parsing approaches")
-    public void localeParsePerformance() throws Exception {
-        TesterRequest req = new TesterRequest();
-        req.addHeader("accept-encoding", "en-gb,en");
+    public void testGetReaderValidEncoding() throws Exception {
+        doTestGetReader("ISO-8859-1", true);
+    }
 
-        long start = System.nanoTime();
 
-        // Takes about 0.3s on a quad core 2.7Ghz 2013 MacBook
-        for (int i = 0; i < 10000000; i++) {
-            req.parseLocales();
-            req.localesParsed = false;
-            req.locales.clear();
+    @Test
+    public void testGetReaderInvalidEncoding() throws Exception {
+        doTestGetReader("X-Invalid", false);
+    }
+
+
+    private void doTestGetReader(String userAgentCharacterEncoding, boolean expect200)
+            throws Exception {
+
+        // Setup Tomcat instance
+        Tomcat tomcat = getTomcatInstance();
+
+        // No file system docBase required
+        Context ctx = getProgrammaticRootContext();
+
+        Tomcat.addServlet(ctx, "servlet", new Bug61264GetReaderServlet());
+        ctx.addServletMappingDecoded("/", "servlet");
+
+        tomcat.start();
+
+        Charset charset = StandardCharsets.ISO_8859_1;
+        try {
+            charset = Charset.forName(userAgentCharacterEncoding);
+        } catch (UnsupportedCharsetException e) {
+            // Ignore - use default set above
+        }
+        byte[] body = "Test".getBytes(charset);
+        ByteChunk bc = new ByteChunk();
+        Map<String,List<String>> reqHeaders = new HashMap<>();
+        reqHeaders.put("Content-Type",
+                Arrays.asList(new String[] {"text/plain;charset=" + userAgentCharacterEncoding}));
+
+        int rc = postUrl(body, "http://localhost:" + getPort() + "/", bc, reqHeaders, null);
+
+        if (expect200) {
+            Assert.assertEquals(200, rc);
+        } else {
+            Assert.assertEquals(500, rc);
+        }
+    }
+
+
+    private static class Bug61264GetReaderServlet extends HttpServlet {
+
+        private static final long serialVersionUID = 1L;
+
+        @Override
+        protected void doGet(HttpServletRequest req, HttpServletResponse resp)
+                throws ServletException, IOException {
+            // This is intended for POST requests
+            resp.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
         }
 
-        long time = System.nanoTime() - start;
+        @Override
+        protected void doPost(HttpServletRequest req, HttpServletResponse resp)
+                throws ServletException, IOException {
+            // Container will handle any errors
+            req.getReader();
+        }
+    }
 
-        System.out.println(time);
+
+    /*
+     * https://bz.apache.org/bugzilla/show_bug.cgi?id=69442
+     */
+    @Test
+    public void testTestParameterMediaTypeLowerCase() throws Exception {
+        // toLowerCase() is unnecessary but keep it in case the constant is changed in the future
+        doTestParameterMediaTypeCase(Globals.CONTENT_TYPE_FORM_URL_ENCODING.toLowerCase(Locale.ENGLISH));
+    }
+
+
+    /*
+     * https://bz.apache.org/bugzilla/show_bug.cgi?id=69442
+     */
+    @Test
+    public void testTestParameterMediaTypeUpperCase() throws Exception {
+        doTestParameterMediaTypeCase(Globals.CONTENT_TYPE_FORM_URL_ENCODING.toUpperCase(Locale.ENGLISH));
+    }
+
+
+    private void doTestParameterMediaTypeCase(String contentType) throws Exception {
+        // Setup Tomcat instance
+        Tomcat tomcat = getTomcatInstance();
+
+        // No file system docBase required
+        Context ctx = getProgrammaticRootContext();
+
+        Tomcat.addServlet(ctx, "servlet", new Bug69442Servlet());
+        ctx.addServletMappingDecoded("/", "servlet");
+
+        tomcat.start();
+
+        ByteChunk bc = new ByteChunk();
+        Map<String,List<String>> reqHeaders = new HashMap<>();
+        reqHeaders.put("Content-Type", Arrays.asList(contentType));
+        postUrl("a=b&c=d".getBytes(), "http://localhost:" + getPort() + "/", bc, reqHeaders, null);
+        String responseBody = bc.toString();
+        Assert.assertTrue(responseBody, responseBody.contains("a=b"));
+        Assert.assertTrue(responseBody, responseBody.contains("c=d"));
+    }
+
+
+    private static class Bug69442Servlet extends HttpServlet {
+
+        private static final long serialVersionUID = 1L;
+
+        @Override
+        protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
+            resp.setContentType("text/plain");
+            resp.setCharacterEncoding(StandardCharsets.UTF_8.name());
+            PrintWriter pw = resp.getWriter();
+            Enumeration<String> names = req.getParameterNames();
+            while (names.hasMoreElements()) {
+                String name = names.nextElement();
+                String[] values = req.getParameterValues(name);
+                pw.println(name + "=" + StringUtils.join(values));
+            }
+        }
     }
 }
